"""
A dataset collects together all data related to a given sample.

This includes a calibration matrix and any XES / RIXS spectra
This could also include any plots generated from the data

Needless to say, this hasn't been written yet...
"""

class DataSet:
  def __init__(self):
    self.name = ""
    self.description = ""

